@@ -14,4 +14,6 @@ from ipy_completers import *
 ip.set_hook('complete_command', apt_completer, re_key = '.*apt-get')
 ip.set_hook('complete_command', svn_completer, str_key = 'svn')
 ip.set_hook('complete_command', hg_completer, str_key = 'hg')
-ip.set_hook('complete_command', bzr_completer, str_key = 'bzr')
+
+# the old bzr completer is deprecated, we recommend ipy_bzr
+#ip.set_hook('complete_command', bzr_completer, str_key = 'bzr')
