@@ -3,8 +3,7 @@
 
 All the matplotlib support code was co-developed with John Hunter,
 matplotlib's author.
-
-$Id: Shell.py 2764 2007-09-15 14:16:21Z darren.dale $"""
+"""
 
 #*****************************************************************************
 #       Copyright (C) 2001-2006 Fernando Perez <fperez@colorado.edu>
@@ -13,10 +12,6 @@ $Id: Shell.py 2764 2007-09-15 14:16:21Z darren.dale $"""
 #  the file COPYING, distributed as part of this software.
 #*****************************************************************************
 
-from IPython import Release
-__author__  = '%s <%s>' % Release.authors['Fernando']
-__license__ = Release.license
-
 # Code begins
 # Stdlib imports
 import __builtin__
@@ -40,11 +35,12 @@ except ImportError:
 # IPython imports
 import IPython
 from IPython import ultraTB, ipapi
+from IPython.Magic import Magic
 from IPython.genutils import Term,warn,error,flag_calls, ask_yes_no
 from IPython.iplib import InteractiveShell
 from IPython.ipmaker import make_IPython
-from IPython.Magic import Magic
 from IPython.ipstruct import Struct
+from IPython.testing import decorators as testdec
 
 # Globals
 # global flag to pass around information about Ctrl-C without exceptions
@@ -59,6 +55,9 @@ MAIN_THREAD_ID = thread.get_ident()
 # Tag when runcode() is active, for exception handling
 CODE_RUN = None
 
+# Default timeout for waiting for multithreaded shells (in seconds)
+GUI_TIMEOUT = 10
+
 #-----------------------------------------------------------------------------
 # This class is trivial now, but I want to have it in to publish a clean
 # interface. Later when the internals are reorganized, code that uses this
@@ -293,17 +292,19 @@ if HAS_CTYPES:
         """raises the exception, performs cleanup if needed"""
         if not inspect.isclass(exctype):
             raise TypeError("Only types can be raised (not instances)")
-        res = ctypes.pythonapi.PyThreadState_SetAsyncExc(tid,
+        # Explicit cast to c_long is necessary for 64-bit support:
+        # See https://bugs.launchpad.net/ipython/+bug/237073
+        res = ctypes.pythonapi.PyThreadState_SetAsyncExc(ctypes.c_long(tid),
                                                          ctypes.py_object(exctype))
         if res == 0:
             raise ValueError("invalid thread id")
         elif res != 1:
-            # """if it returns a number greater than one, you're in trouble, 
-            # and you should call it again with exc=NULL to revert the effect"""
+            # If it returns a number greater than one, you're in trouble, 
+            # and you should call it again with exc=NULL to revert the effect
             ctypes.pythonapi.PyThreadState_SetAsyncExc(tid, 0)
             raise SystemError("PyThreadState_SetAsyncExc failed")
 
-    def sigint_handler (signum,stack_frame):
+    def sigint_handler(signum,stack_frame):
         """Sigint handler for threaded apps.
 
         This is a horrible hack to pass information about SIGINT _without_
@@ -322,7 +323,7 @@ if HAS_CTYPES:
             Term.cout.flush()
 
 else:
-    def sigint_handler (signum,stack_frame):
+    def sigint_handler(signum,stack_frame):
         """Sigint handler for threaded apps.
 
         This is a horrible hack to pass information about SIGINT _without_
@@ -352,44 +353,46 @@ class MTInteractiveShell(InteractiveShell):
     isthreaded = True
 
     def __init__(self,name,usage=None,rc=Struct(opts=None,args=None),
-                 user_ns=None,user_global_ns=None,banner2='',**kw):
+                 user_ns=None,user_global_ns=None,banner2='',
+                 gui_timeout=GUI_TIMEOUT,**kw):
         """Similar to the normal InteractiveShell, but with threading control"""
         
         InteractiveShell.__init__(self,name,usage,rc,user_ns,
                                   user_global_ns,banner2)
 
-        # Locking control variable.  We need to use a norma lock, not an RLock
-        # here.  I'm not exactly sure why, it seems to me like it should be
-        # the opposite, but we deadlock with an RLock.  Puzzled...
-        self.thread_ready = threading.Condition(threading.Lock())
+        # Timeout we wait for GUI thread
+        self.gui_timeout = gui_timeout
 
-        # A queue to hold the code to be executed.  A scalar variable is NOT
-        # enough, because uses like macros cause reentrancy.
+        # A queue to hold the code to be executed. 
         self.code_queue = Queue.Queue()
 
         # Stuff to do at closing time
-        self._kill = False
-        on_kill = kw.get('on_kill')
-        if on_kill is None:
-            on_kill = []
+        self._kill = None
+        on_kill = kw.get('on_kill', [])
         # Check that all things to kill are callable:
         for t in on_kill:
             if not callable(t):
                 raise TypeError,'on_kill must be a list of callables'
         self.on_kill = on_kill
-
+        # thread identity of the "worker thread" (that may execute code directly)
+        self.worker_ident = None
+        
     def runsource(self, source, filename="<input>", symbol="single"):
         """Compile and run some source in the interpreter.
 
         Modified version of code.py's runsource(), to handle threading issues.
         See the original for full docstring details."""
-
+                
         global KBINT
         
         # If Ctrl-C was typed, we reset the flag and return right away
         if KBINT:
             KBINT = False
             return False
+
+        if self._kill:
+            # can't queue new code if we are being killed
+            return True
         
         try:
             code = self.compile(source, filename, symbol)
@@ -402,33 +405,41 @@ class MTInteractiveShell(InteractiveShell):
             # Case 2
             return True
 
+        # shortcut - if we are in worker thread, or the worker thread is not
+        # running, execute directly (to allow recursion and prevent deadlock if
+        # code is run early in IPython construction)
+        
+        if (self.worker_ident is None
+            or self.worker_ident == thread.get_ident() ):
+            InteractiveShell.runcode(self,code)
+            return False
+
         # Case 3
         # Store code in queue, so the execution thread can handle it.
 
-        # Note that with macros and other applications, we MAY re-enter this
-        # section, so we have to acquire the lock with non-blocking semantics,
-        # else we deadlock.
-        got_lock = self.thread_ready.acquire(False)
-        self.code_queue.put(code)
-        if got_lock:
-            self.thread_ready.wait()  # Wait until processed in timeout interval
-            self.thread_ready.release()
-
+        completed_ev, received_ev = threading.Event(), threading.Event() 
+        
+        self.code_queue.put((code,completed_ev, received_ev))
+        # first make sure the message was received, with timeout
+        received_ev.wait(self.gui_timeout)
+        if not received_ev.isSet():
+            # the mainloop is dead, start executing code directly
+            print "Warning: Timeout for mainloop thread exceeded"
+            print "switching to nonthreaded mode (until mainloop wakes up again)"
+            self.worker_ident = None
+        else:
+            completed_ev.wait()
         return False
 
     def runcode(self):
         """Execute a code object.
 
         Multithreaded wrapper around IPython's runcode()."""
-
-        global CODE_RUN
-
-        # Exceptions need to be raised differently depending on which thread is
-        # active
-        CODE_RUN = True
         
-        # lock thread-protected stuff
-        got_lock = self.thread_ready.acquire(False)
+        global CODE_RUN
+        
+        # we are in worker thread, stash out the id for runsource() 
+        self.worker_ident = thread.get_ident()
 
         if self._kill:
             print >>Term.cout, 'Closing threads...',
@@ -436,6 +447,9 @@ class MTInteractiveShell(InteractiveShell):
             for tokill in self.on_kill:
                 tokill()
             print >>Term.cout, 'Done.'
+            # allow kill() to return
+            self._kill.set()
+            return True
 
         # Install sigint handler.  We do it every time to ensure that if user
         # code modifies it, we restore our own handling.
@@ -448,32 +462,44 @@ class MTInteractiveShell(InteractiveShell):
 
         # Flush queue of pending code by calling the run methood of the parent
         # class with all items which may be in the queue.
+        code_to_run = None
         while 1:
             try:
-                code_to_run = self.code_queue.get_nowait()
+                code_to_run, completed_ev, received_ev = self.code_queue.get_nowait()                
             except Queue.Empty:
                 break
-            if got_lock:
-                self.thread_ready.notify()
-                InteractiveShell.runcode(self,code_to_run)
-            else:
-                break
+            received_ev.set()
             
-        # We're done with thread-protected variables
-        if got_lock:
-            self.thread_ready.release()
-
-        # We're done...
-        CODE_RUN = False
+            # Exceptions need to be raised differently depending on which
+            # thread is active.  This convoluted try/except is only there to
+            # protect against asynchronous exceptions, to ensure that a KBINT
+            # at the wrong time doesn't deadlock everything.  The global
+            # CODE_TO_RUN is set to true/false as close as possible to the
+            # runcode() call, so that the KBINT handler is correctly informed.
+            try:
+                try:
+                   CODE_RUN = True
+                   InteractiveShell.runcode(self,code_to_run)
+                except KeyboardInterrupt:
+                   print "Keyboard interrupted in mainloop"
+                   while not self.code_queue.empty():
+                      code, ev1,ev2 = self.code_queue.get_nowait()
+                      ev1.set()
+                      ev2.set()                      
+                   break
+            finally:
+                CODE_RUN = False
+                # allow runsource() return from wait
+                completed_ev.set()
+                
+        
         # This MUST return true for gtk threading to work
         return True
 
     def kill(self):
         """Kill the thread, returning when it has been shut down."""
-        got_lock = self.thread_ready.acquire(False)
-        self._kill = True
-        if got_lock:
-            self.thread_ready.release()
+        self._kill = threading.Event()
+        self._kill.wait()
 
 class MatplotlibShellBase:
     """Mixin class to provide the necessary modifications to regular IPython
@@ -482,7 +508,7 @@ class MatplotlibShellBase:
     Given Python's MRO, this should be used as the FIRST class in the
     inheritance hierarchy, so that it overrides the relevant methods."""
     
-    def _matplotlib_config(self,name,user_ns):
+    def _matplotlib_config(self,name,user_ns,user_global_ns=None):
         """Return items needed to setup the user's shell with matplotlib"""
 
         # Initialize matplotlib to interactive mode always
@@ -536,17 +562,28 @@ class MatplotlibShellBase:
         self.pylab.draw_if_interactive = flag_calls(self.pylab.draw_if_interactive)
 
         # Build a user namespace initialized with matplotlib/matlab features.
-        user_ns = IPython.ipapi.make_user_ns(user_ns)
+        user_ns, user_global_ns = IPython.ipapi.make_user_namespaces(user_ns,
+            user_global_ns)
 
-        exec ("import matplotlib\n"
-              "import matplotlib.pylab as pylab\n") in user_ns
+        # Import numpy as np/pyplot as plt are conventions we're trying to
+        # somewhat standardize on.  Making them available to users by default
+        # will greatly help this. 
+        exec ("import numpy\n"
+              "import numpy as np\n"
+              "import matplotlib\n"
+              "import matplotlib.pylab as pylab\n"
+              "try:\n"
+              "    import matplotlib.pyplot as plt\n"
+              "except ImportError:\n"
+              "    pass\n"
+              ) in user_ns
         
         # Build matplotlib info banner
         b="""
   Welcome to pylab, a matplotlib-based Python environment.
   For more information, type 'help(pylab)'.
 """
-        return user_ns,b
+        return user_ns,user_global_ns,b
 
     def mplot_exec(self,fname,*where,**kw):
         """Execute a matplotlib script.
@@ -568,7 +605,8 @@ class MatplotlibShellBase:
         # if a backend switch was performed, reverse it now
         if self.mpl_use._called:
             self.matplotlib.rcParams['backend'] = self.mpl_backend
-        
+
+    @testdec.skip_doctest
     def magic_run(self,parameter_s=''):
         Magic.magic_run(self,parameter_s,runner=self.mplot_exec)
 
@@ -586,7 +624,7 @@ class MatplotlibShell(MatplotlibShellBase,InteractiveShell):
 
     def __init__(self,name,usage=None,rc=Struct(opts=None,args=None),
                  user_ns=None,user_global_ns=None,**kw):
-        user_ns,b2 = self._matplotlib_config(name,user_ns)
+        user_ns,user_global_ns,b2 = self._matplotlib_config(name,user_ns,user_global_ns)
         InteractiveShell.__init__(self,name,usage,rc,user_ns,user_global_ns,
                                   banner2=b2,**kw)
 
@@ -595,7 +633,7 @@ class MatplotlibMTShell(MatplotlibShellBase,MTInteractiveShell):
 
     def __init__(self,name,usage=None,rc=Struct(opts=None,args=None),
                  user_ns=None,user_global_ns=None, **kw):
-        user_ns,b2 = self._matplotlib_config(name,user_ns)
+        user_ns,user_global_ns,b2 = self._matplotlib_config(name,user_ns,user_global_ns)
         MTInteractiveShell.__init__(self,name,usage,rc,user_ns,user_global_ns,
                                     banner2=b2,**kw)
 
@@ -735,6 +773,17 @@ class IPShellGTK(IPThread):
                  debug=1,shell_class=MTInteractiveShell):
 
         import gtk
+        # Check for set_interactive, coming up in new pygtk.
+        # Disable it so that this code works, but notify 
+        # the user that he has a better option as well.
+        # XXX TODO better support when set_interactive is released
+        try:
+            gtk.set_interactive(False)
+            print "Your PyGtk has set_interactive(), so you can use the"
+            print "more stable single-threaded Gtk mode."
+            print "See https://bugs.launchpad.net/ipython/+bug/270856"
+        except AttributeError:
+            pass
         
         self.gtk = gtk
         self.gtk_mainloop = hijack_gtk()
@@ -972,6 +1021,12 @@ class IPShellQt4(IPThread):
 
         from PyQt4 import QtCore, QtGui
 
+        try:
+            # present in PyQt4-4.2.1 or later
+            QtCore.pyqtRemoveInputHook()
+        except AttributeError:
+            pass
+
         if QtCore.PYQT_VERSION_STR == '4.3':
             warn('''PyQt4 version 4.3 detected.
 If you experience repeated threading warnings, please update PyQt4.
@@ -1035,7 +1090,9 @@ def _load_pylab(user_ns):
     
     ip = IPython.ipapi.get()
     if ip.options.pylab_import_all:
-        exec "from matplotlib.pylab import *" in user_ns
+        ip.ex("from matplotlib.pylab import *")
+        ip.IP.user_config_ns.update(ip.user_ns)
+        
 
 class IPShellMatplotlib(IPShell):
     """Subclass IPShell with MatplotlibShell as the internal shell.
@@ -1092,31 +1149,85 @@ class IPShellMatplotlibQt4(IPShellQt4):
 #-----------------------------------------------------------------------------
 # Factory functions to actually start the proper thread-aware shell
 
-def _matplotlib_shell_class():
-    """Factory function to handle shell class selection for matplotlib.
+def _select_shell(argv):
+    """Select a shell from the given argv vector.
 
-    The proper shell class to use depends on the matplotlib backend, since
-    each backend requires a different threading strategy."""
+    This function implements the threading selection policy, allowing runtime
+    control of the threading mode, both for general users and for matplotlib.
 
-    try:
-        import matplotlib
-    except ImportError:
-        error('matplotlib could NOT be imported!  Starting normal IPython.')
-        sh_class = IPShell
-    else:
-        backend = matplotlib.rcParams['backend']
-        if backend.startswith('GTK'):
-            sh_class = IPShellMatplotlibGTK
-        elif backend.startswith('WX'):
-            sh_class = IPShellMatplotlibWX
-        elif backend.startswith('Qt4'):
-            sh_class = IPShellMatplotlibQt4
-        elif backend.startswith('Qt'):
-            sh_class = IPShellMatplotlibQt
+    Return:
+      Shell class to be instantiated for runtime operation.
+    """
+    
+    global USE_TK
+
+    mpl_shell = {'gthread' : IPShellMatplotlibGTK,
+                 'wthread' : IPShellMatplotlibWX,
+                 'qthread' : IPShellMatplotlibQt,
+                 'q4thread' : IPShellMatplotlibQt4,
+                 'tkthread' : IPShellMatplotlib,  # Tk is built-in
+                 }
+
+    th_shell = {'gthread' : IPShellGTK,
+                'wthread' : IPShellWX,
+                'qthread' : IPShellQt,
+                'q4thread' : IPShellQt4,
+                'tkthread' : IPShell, # Tk is built-in
+                }
+
+    backends = {'gthread' : 'GTKAgg',
+                'wthread' : 'WXAgg',
+                'qthread' : 'QtAgg',
+                'q4thread' :'Qt4Agg',
+                'tkthread' :'TkAgg',
+                }
+
+    all_opts = set(['tk','pylab','gthread','qthread','q4thread','wthread',
+                    'tkthread'])
+    user_opts = set([s.replace('-','') for s in argv[:3]])
+    special_opts = user_opts & all_opts            
+
+    if 'tk' in special_opts:
+        USE_TK = True
+        special_opts.remove('tk')
+
+    if 'pylab' in special_opts:
+
+        try:
+            import matplotlib
+        except ImportError:
+            error('matplotlib could NOT be imported!  Starting normal IPython.')
+            return IPShell
+        
+        special_opts.remove('pylab')
+        # If there's any option left, it means the user wants to force the
+        # threading backend, else it's auto-selected from the rc file
+        if special_opts:
+            th_mode = special_opts.pop()
+            matplotlib.rcParams['backend'] = backends[th_mode]
         else:
-            sh_class = IPShellMatplotlib
-    #print 'Using %s with the %s backend.' % (sh_class,backend) # dbg
-    return sh_class
+            backend = matplotlib.rcParams['backend']
+            if backend.startswith('GTK'):
+                th_mode = 'gthread'
+            elif backend.startswith('WX'):
+                th_mode = 'wthread'
+            elif backend.startswith('Qt4'):
+                th_mode = 'q4thread'
+            elif backend.startswith('Qt'):
+                th_mode = 'qthread'
+            else:
+                # Any other backend, use plain Tk
+                th_mode = 'tkthread'
+                
+        return mpl_shell[th_mode]
+    else:
+        # No pylab requested, just plain threads
+        try:
+            th_mode = special_opts.pop()
+        except KeyError:
+            th_mode = 'tkthread'
+        return th_shell[th_mode]
+
 
 # This is the one which should be called by external code.
 def start(user_ns = None):
@@ -1126,29 +1237,7 @@ def start(user_ns = None):
     based on the user's threading choice.  Such a selector is needed because
     different GUI toolkits require different thread handling details."""
 
-    global USE_TK
-    # Crude sys.argv hack to extract the threading options.
-    argv = sys.argv
-    if len(argv) > 1:
-        if len(argv) > 2:
-            arg2 = argv[2]
-            if arg2.endswith('-tk'):
-                USE_TK = True
-        arg1 = argv[1]
-        if arg1.endswith('-gthread'):
-            shell = IPShellGTK
-        elif arg1.endswith( '-qthread' ):
-            shell = IPShellQt
-        elif arg1.endswith( '-q4thread' ):
-            shell = IPShellQt4
-        elif arg1.endswith('-wthread'):
-            shell = IPShellWX
-        elif arg1.endswith('-pylab'):
-            shell = _matplotlib_shell_class()
-        else:
-            shell = IPShell
-    else:
-        shell = IPShell
+    shell = _select_shell(sys.argv)
     return shell(user_ns = user_ns)
 
 # Some aliases for backwards compatibility
