@@ -6,7 +6,6 @@ upstream and were accepted as of Python 2.3, but we need a lot more
 functionality specific to IPython, so this module will continue to live as an
 IPython-specific utility.
 
----------------------------------------------------------------------------
 Original rlcompleter documentation:
 
 This requires the latest extension to the readline module (the
@@ -108,13 +107,6 @@ class Completer:
         readline.set_completer(Completer(my_namespace).complete)
         """
 
-        # some minimal strict typechecks.  For some core data structures, I
-        # want actual basic python types, not just anything that looks like
-        # one.  This is especially true for namespaces.
-        for ns in (namespace,global_namespace):
-            if ns is not None and type(ns) != types.DictType:
-                raise TypeError,'namespace must be a dictionary'
-
         # Don't bind to namespace quite yet, but flag whether the user wants a
         # specific namespace or to use __main__.__dict__. This will allow us
         # to bind to __main__.__dict__ at completion time, not now.
@@ -321,7 +313,10 @@ class IPCompleter(Completer):
         # don't want to treat as delimiters in filename matching
         # when escaped with backslash
 
-        protectables = ' '
+        if sys.platform == 'win32':
+            protectables = ' '
+        else:
+            protectables = ' ()'
 
         if text.startswith('!'):
             text = text[1:]
