@@ -1,4 +1,4 @@
-''' IPython customization API
+"""IPython customization API
 
 Your one-stop module for configuring & extending ipython
 
@@ -24,55 +24,54 @@ That way the module is imported at startup and you can have all your
 personal configuration (as opposed to boilerplate ipythonrc-PROFILENAME 
 stuff) in there. 
 
------------------------------------------------
 import IPython.ipapi
 ip = IPython.ipapi.get()
 
 def ankka_f(self, arg):
-    print "Ankka",self,"says uppercase:",arg.upper()
+    print 'Ankka',self,'says uppercase:',arg.upper()
 
-ip.expose_magic("ankka",ankka_f)
+ip.expose_magic('ankka',ankka_f)
 
 ip.magic('alias sayhi echo "Testing, hi ok"')
 ip.magic('alias helloworld echo "Hello world"')
 ip.system('pwd')
 
 ip.ex('import re')
-ip.ex("""
+ip.ex('''
 def funcci(a,b):
     print a+b
 print funcci(3,4)
-""")
-ip.ex("funcci(348,9)")
+''')
+ip.ex('funcci(348,9)')
 
 def jed_editor(self,filename, linenum=None):
-    print "Calling my own editor, jed ... via hook!"
+    print 'Calling my own editor, jed ... via hook!'
     import os
     if linenum is None: linenum = 0
     os.system('jed +%d %s' % (linenum, filename))
-    print "exiting jed"
+    print 'exiting jed'
 
 ip.set_hook('editor',jed_editor)
 
 o = ip.options
 o.autocall = 2  # FULL autocall mode
 
-print "done!"
-'''
+print 'done!'
+"""
+
+#-----------------------------------------------------------------------------
+# Modules and globals
 
 # stdlib imports
 import __builtin__
 import sys
 
-try: # Python 2.3 compatibility
-    set
-except NameError:
-    import sets
-    set = sets.Set
+# contains the most recently instantiated IPApi
+_RECENT_IP = None
+
+#-----------------------------------------------------------------------------
+# Code begins
 
-# our own
-#from IPython.genutils import warn,error
- 
 class TryNext(Exception):
     """Try next hook exception.
      
@@ -86,6 +85,7 @@ class TryNext(Exception):
         self.args = args
         self.kwargs = kwargs
 
+
 class UsageError(Exception):
     """ Error in magic function arguments, etc.
     
@@ -93,6 +93,7 @@ class UsageError(Exception):
     nevertheless interrupt a macro / batch file.   
     """
 
+
 class IPyAutocall:
     """ Instances of this class are always autocalled
     
@@ -109,8 +110,6 @@ class IPyAutocall:
         self._ip = ip
     
 
-# contains the most recently instantiated IPApi
-
 class IPythonNotRunning:
     """Dummy do-nothing class.
 
@@ -144,8 +143,6 @@ class IPythonNotRunning:
         """Dummy function, which doesn't do anything and emits no warnings."""
         pass
 
-_recent = None
-
 
 def get(allow_dummy=False,dummy_warn=True):
     """Get an IPApi object.
@@ -159,12 +156,13 @@ def get(allow_dummy=False,dummy_warn=True):
     can be imported as normal modules. You can then direct all the
     configuration operations against the returned object.
     """
-    global _recent
-    if allow_dummy and not _recent:
-        _recent = IPythonNotRunning(dummy_warn)
-    return _recent
+    global _RECENT_IP
+    if allow_dummy and not _RECENT_IP:
+        _RECENT_IP = IPythonNotRunning(dummy_warn)
+    return _RECENT_IP
 
-class IPApi:
+
+class IPApi(object):
     """ The actual API class for configuring IPython 
     
     You should do all of the IPython configuration by getting an IPApi object
@@ -173,6 +171,8 @@ class IPApi:
     
     def __init__(self,ip):
         
+        global _RECENT_IP
+
         # All attributes exposed here are considered to be the public API of
         # IPython.  As needs dictate, some of these may be wrapped as
         # properties.
@@ -200,8 +200,7 @@ class IPApi:
 
         self.dbg = DebugTools(self)
         
-        global _recent
-        _recent = self
+        _RECENT_IP = self
 
     # Use a property for some things which are added to the instance very
     # late.  I don't have time right now to disentangle the initialization
@@ -217,8 +216,8 @@ class IPApi:
         """All configurable variables."""
         
         # catch typos by disabling new attribute creation. If new attr creation
-        # is in fact wanted (e.g. when exposing new options), do allow_new_attr(True) 
-        # for the received rc struct.
+        # is in fact wanted (e.g. when exposing new options), do
+        # allow_new_attr(True) for the received rc struct.
         
         self.IP.rc.allow_new_attr(False)
         return self.IP.rc
@@ -226,22 +225,23 @@ class IPApi:
     options = property(get_options,None,None,get_options.__doc__)
     
     def expose_magic(self,magicname, func):
-        ''' Expose own function as magic function for ipython 
+        """Expose own function as magic function for ipython 
     
         def foo_impl(self,parameter_s=''):
-            """My very own magic!. (Use docstrings, IPython reads them)."""
-            print 'Magic function. Passed parameter is between < >: <'+parameter_s+'>'
+            'My very own magic!. (Use docstrings, IPython reads them).'
+            print 'Magic function. Passed parameter is between < >:'
+            print '<%s>' % parameter_s
             print 'The self object is:',self
     
-        ipapi.expose_magic("foo",foo_impl)
-        '''
-                
+        ipapi.expose_magic('foo',foo_impl)
+        """
+        
         import new
         im = new.instancemethod(func,self.IP, self.IP.__class__)
         old = getattr(self.IP, "magic_" + magicname, None)
         if old:
-            self.dbg.debug_stack("Magic redefinition '%s', old %s" % (magicname,
-                                 old))
+            self.dbg.debug_stack("Magic redefinition '%s', old %s" %
+                                 (magicname,old) )
             
         setattr(self.IP, "magic_" + magicname, im)
     
@@ -262,11 +262,50 @@ class IPApi:
         
         Takes either all lines in one string or list of lines.
         """
-        if isinstance(lines,basestring):
-            self.IP.runlines(lines)
-        else:
-            self.IP.runlines('\n'.join(lines))
 
+        def cleanup_ipy_script(script):
+            """ Make a script safe for _ip.runlines() 
+            
+            - Removes empty lines Suffixes all indented blocks that end with
+            - unindented lines with empty lines
+            """
+            
+            res = []
+            lines = script.splitlines()
+
+            level = 0
+            for l in lines:
+                lstripped = l.lstrip()
+                stripped = l.strip()                
+                if not stripped:
+                    continue
+                newlevel = len(l) - len(lstripped)
+                def is_secondary_block_start(s):
+                    if not s.endswith(':'):
+                        return False
+                    if (s.startswith('elif') or 
+                        s.startswith('else') or 
+                        s.startswith('except') or
+                        s.startswith('finally')):
+                        return True
+                        
+                if level > 0 and newlevel == 0 and \
+                       not is_secondary_block_start(stripped): 
+                    # add empty line
+                    res.append('')
+                    
+                res.append(l)
+                level = newlevel
+            return '\n'.join(res) + '\n'
+        
+        if isinstance(lines,basestring):
+            script = lines            
+        else:
+            script = '\n'.join(lines)
+        clean=cleanup_ipy_script(script)
+        # print "_ip.runlines() script:\n",clean # dbg
+        self.IP.runlines(clean)
+        
     def to_user_ns(self,vars, interactive = True):
         """Inject a group of variables into the IPython user namespace.
 
@@ -354,7 +393,6 @@ class IPApi:
             for name,val in vdict.iteritems():
                 config_ns[name] = val                    
 
-
     def expand_alias(self,line):
         """ Expand an alias in the command line 
         
@@ -387,11 +425,9 @@ class IPApi:
 
         self.dbg.check_hotname(name)
         
-
         if name in self.IP.alias_table:
-            self.dbg.debug_stack("Alias redefinition: '%s' => '%s' (old '%s')" %
-                             (name, cmd, self.IP.alias_table[name]))
-            
+            self.dbg.debug_stack("Alias redefinition: '%s' => '%s' (old '%s')"
+                                 % (name, cmd, self.IP.alias_table[name]))
 
         if callable(cmd):
             self.IP.alias_table[name] = cmd
@@ -402,8 +438,8 @@ class IPApi:
         if isinstance(cmd,basestring):
             nargs = cmd.count('%s')
             if nargs>0 and cmd.find('%l')>=0:
-                raise Exception('The %s and %l specifiers are mutually exclusive '
-                                'in alias definitions.')
+                raise Exception('The %s and %l specifiers are mutually '
+                                'exclusive in alias definitions.')
                   
             self.IP.alias_table[name] = (nargs,cmd)
             return
@@ -456,8 +492,8 @@ class IPApi:
         
         - run init_ipython(ip)
         - run ipython_firstrun(ip)
-        
         """
+        
         if mod in self.extensions:
             # just to make sure we don't init it twice
             # note that if you 'load' a module that has already been
@@ -507,7 +543,8 @@ class DebugTools:
         if name in self.hotnames:
             self.debug_stack( "HotName '%s' caught" % name)
 
-def launch_new_instance(user_ns = None):
+
+def launch_new_instance(user_ns = None,shellclass = None):
     """ Make and start a new ipython instance.
     
     This can be called even without having an already initialized 
@@ -516,7 +553,7 @@ def launch_new_instance(user_ns = None):
     This is also used as the egg entry point for the 'ipython' script.
     
     """
-    ses = make_session(user_ns)
+    ses = make_session(user_ns,shellclass)
     ses.mainloop()
 
 
@@ -526,19 +563,22 @@ def make_user_ns(user_ns = None):
     This builds a dict with the minimal information needed to operate as a
     valid IPython user namespace, which you can pass to the various embedding
     classes in ipython.
+
+    This API is currently deprecated. Use ipapi.make_user_namespaces() instead
+    to make both the local and global namespace objects simultaneously.
+
+    :Parameters:
+        user_ns : dict-like, optional
+            The current user namespace. The items in this namespace should be
+            included in the output. If None, an appropriate blank namespace
+            should be created.
+
+    :Returns:
+        A dictionary-like object to be used as the local namespace of the
+        interpreter.
     """
 
-    if user_ns is None:
-        # Set __name__ to __main__ to better match the behavior of the
-        # normal interpreter.
-        user_ns = {'__name__'     :'__main__',
-                   '__builtins__' : __builtin__,
-                   }
-    else:
-        user_ns.setdefault('__name__','__main__')
-        user_ns.setdefault('__builtins__',__builtin__)
-
-    return user_ns
+    raise NotImplementedError
 
 
 def make_user_global_ns(ns = None):
@@ -546,13 +586,88 @@ def make_user_global_ns(ns = None):
 
     Similar to make_user_ns(), but global namespaces are really only needed in
     embedded applications, where there is a distinction between the user's
-    interactive namespace and the global one where ipython is running."""
+    interactive namespace and the global one where ipython is running.
 
-    if ns is None: ns = {}
-    return ns
+    This API is currently deprecated. Use ipapi.make_user_namespaces() instead
+    to make both the local and global namespace objects simultaneously.
+
+    :Parameters:
+        ns : dict, optional
+            The current user global namespace. The items in this namespace
+            should be included in the output. If None, an appropriate blank
+            namespace should be created.
+
+    :Returns:
+        A true dict to be used as the global namespace of the interpreter.
+    """
+
+    raise NotImplementedError
+
+# Record the true objects in order to be able to test if the user has overridden
+# these API functions.
+_make_user_ns = make_user_ns
+_make_user_global_ns = make_user_global_ns
 
 
-def make_session(user_ns = None):
+def make_user_namespaces(user_ns = None,user_global_ns = None):
+    """Return a valid local and global user interactive namespaces.
+
+    This builds a dict with the minimal information needed to operate as a
+    valid IPython user namespace, which you can pass to the various embedding
+    classes in ipython.  The default implementation returns the same dict for
+    both the locals and the globals to allow functions to refer to variables in
+    the namespace.  Customized implementations can return different dicts.  The
+    locals dictionary can actually be anything following the basic mapping
+    protocol of a dict, but the globals dict must be a true dict, not even
+    a subclass.  It is recommended that any custom object for the locals
+    namespace synchronize with the globals dict somehow.
+
+    Raises TypeError if the provided globals namespace is not a true dict.
+
+    :Parameters:
+        user_ns : dict-like, optional
+            The current user namespace. The items in this namespace should be
+            included in the output. If None, an appropriate blank namespace
+            should be created.
+        user_global_ns : dict, optional
+            The current user global namespace. The items in this namespace
+            should be included in the output. If None, an appropriate blank
+            namespace should be created.
+
+    :Returns:
+        A tuple pair of dictionary-like object to be used as the local namespace
+        of the interpreter and a dict to be used as the global namespace.
+    """
+
+    if user_ns is None:
+        if make_user_ns is not _make_user_ns:
+            # Old API overridden.
+            # FIXME: Issue DeprecationWarning, or just let the old API live on?
+            user_ns = make_user_ns(user_ns)
+        else:
+            # Set __name__ to __main__ to better match the behavior of the
+            # normal interpreter.
+            user_ns = {'__name__'     :'__main__',
+                       '__builtins__' : __builtin__,
+                       }
+    else:
+        user_ns.setdefault('__name__','__main__')
+        user_ns.setdefault('__builtins__',__builtin__)
+
+    if user_global_ns is None:
+        if make_user_global_ns is not _make_user_global_ns:
+            # Old API overridden.
+            user_global_ns = make_user_global_ns(user_global_ns)
+        else:
+            user_global_ns = user_ns
+    if type(user_global_ns) is not dict:
+        raise TypeError("user_global_ns must be a true dict; got %r"
+            % type(user_global_ns))
+
+    return user_ns, user_global_ns
+
+
+def make_session(user_ns = None, shellclass = None):
     """Makes, but does not launch an IPython session.
     
     Later on you can call obj.mainloop() on the returned object.
@@ -565,5 +680,6 @@ def make_session(user_ns = None):
     WARNING: This should *not* be run when a session exists already."""
 
     import IPython.Shell
-    return IPython.Shell.start(user_ns)
-
+    if shellclass is None:
+        return IPython.Shell.start(user_ns)
+    return shellclass(user_ns = user_ns)
