@@ -20,6 +20,7 @@ import sys
 import cPickle as pickle
 from types import FunctionType
 import linecache
+import warnings
 
 from twisted.internet import reactor
 from twisted.python import components, log
@@ -90,14 +91,13 @@ class PendingResult(object):
     
     A user should not create a `PendingResult` instance by hand.
     
-    Methods
-    =======
+    Methods:
     
     * `get_result`
     * `add_callback`
     
-    Properties
-    ==========
+    Properties:
+    
     * `r`
     """
     
@@ -389,6 +389,14 @@ def strip_whitespace(source):
 #-------------------------------------------------------------------------------
 
 
+_prop_warn = """\
+
+We are currently refactoring the task dependency system.  This might
+involve the removal of this method and other methods related to engine
+properties.  Please see the docstrings for IPython.kernel.TaskRejectError 
+for more information."""
+
+
 class IFullBlockingMultiEngineClient(Interface):
     pass
 
@@ -730,22 +738,27 @@ class FullBlockingMultiEngineClient(InteractiveMultiEngineClient):
         return self._blockFromThread(self.smultiengine.queue_status, targets=targets, block=block)
     
     def set_properties(self, properties, targets=None, block=None):
+        warnings.warn(_prop_warn)
         targets, block = self._findTargetsAndBlock(targets, block)
         return self._blockFromThread(self.smultiengine.set_properties, properties, targets=targets, block=block)
     
     def get_properties(self, keys=None, targets=None, block=None):
+        warnings.warn(_prop_warn)
         targets, block = self._findTargetsAndBlock(targets, block)
         return self._blockFromThread(self.smultiengine.get_properties, keys, targets=targets, block=block)
     
     def has_properties(self, keys, targets=None, block=None):
+        warnings.warn(_prop_warn)
         targets, block = self._findTargetsAndBlock(targets, block)
         return self._blockFromThread(self.smultiengine.has_properties, keys, targets=targets, block=block)
     
     def del_properties(self, keys, targets=None, block=None):
+        warnings.warn(_prop_warn)
         targets, block = self._findTargetsAndBlock(targets, block)
         return self._blockFromThread(self.smultiengine.del_properties, keys, targets=targets, block=block)
     
     def clear_properties(self, targets=None, block=None):
+        warnings.warn(_prop_warn)
         targets, block = self._findTargetsAndBlock(targets, block)
         return self._blockFromThread(self.smultiengine.clear_properties, targets=targets, block=block)
     
@@ -885,7 +898,62 @@ class FullBlockingMultiEngineClient(InteractiveMultiEngineClient):
         targets, block = self._findTargetsAndBlock(targets, block)
         return self._blockFromThread(self.smultiengine.run, filename,
             targets=targets, block=block)
+    
+    def benchmark(self, push_size=10000):
+        """
+        Run performance benchmarks for the current IPython cluster.
+        
+        This method tests both the latency of sending command and data to the
+        engines as well as the throughput of sending large objects to the
+        engines using push.  The latency is measured by having one or more
+        engines execute the command 'pass'.  The throughput is measure by 
+        sending an NumPy array of size `push_size` to one or more engines.
+        
+        These benchmarks will vary widely on different hardware and networks
+        and thus can be used to get an idea of the performance characteristics
+        of a particular configuration of an IPython controller and engines.
+        
+        This function is not testable within our current testing framework.
+        """
+        import timeit, __builtin__
+        __builtin__._mec_self = self
+        benchmarks = {}
+        repeat = 3
+        count = 10
 
+        timer = timeit.Timer('_mec_self.execute("pass",0)')
+        result = 1000*min(timer.repeat(repeat,count))/count
+        benchmarks['single_engine_latency'] = (result,'msec')
+
+        timer = timeit.Timer('_mec_self.execute("pass")')
+        result = 1000*min(timer.repeat(repeat,count))/count
+        benchmarks['all_engine_latency'] = (result,'msec')
+
+        try:
+            import numpy as np
+        except:
+            pass
+        else:
+            timer = timeit.Timer(
+                "_mec_self.push(d)",
+                "import numpy as np; d = dict(a=np.zeros(%r,dtype='float64'))" % push_size
+            )
+            result = min(timer.repeat(repeat,count))/count
+            benchmarks['all_engine_push'] = (1e-6*push_size*8/result, 'MB/sec')
+
+        try:
+            import numpy as np
+        except:
+            pass
+        else:
+            timer = timeit.Timer(
+                "_mec_self.push(d,0)",
+                "import numpy as np; d = dict(a=np.zeros(%r,dtype='float64'))" % push_size
+            )
+            result = min(timer.repeat(repeat,count))/count
+            benchmarks['single_engine_push'] = (1e-6*push_size*8/result, 'MB/sec')
+
+        return benchmarks
 
 
 components.registerAdapter(FullBlockingMultiEngineClient,
