@@ -16,6 +16,9 @@ __docformat__ = "restructuredtext en"
 # Imports
 #-----------------------------------------------------------------------------
 
+# Tell nose to skip the testing of this module
+__test__ = {}
+
 import copy, time
 from types import FunctionType
 
@@ -411,7 +414,7 @@ class ResultNS(object):
     This can be a bad idea, as it may corrupt standard behavior of the
     ns object.
     
-    Example
+    Examples
     --------
     
     >>> ns = ResultNS({'a':17,'foo':range(3)})
