@@ -23,15 +23,15 @@ __docformat__ = "restructuredtext en"
 # Imports
 #-------------------------------------------------------------------------------
 
-try:
-    from twisted.application.service import IService
-    from IPython.kernel.controllerservice import ControllerService
-    from IPython.kernel.tests import multienginetest as met
-    from controllertest import IControllerCoreTestCase
-    from IPython.testing.util import DeferredTestCase
-except ImportError:
-    import nose
-    raise nose.SkipTest("This test requires zope.interface, Twisted and Foolscap")
+# Tell nose to skip this module
+__test__ = {}
+
+from twisted.application.service import IService
+from IPython.kernel.controllerservice import ControllerService
+from IPython.kernel.tests import multienginetest as met
+from controllertest import IControllerCoreTestCase
+from IPython.testing.util import DeferredTestCase
+
 
 class BasicControllerServiceTest(DeferredTestCase,
     IControllerCoreTestCase):
