@@ -13,29 +13,18 @@ The code in this file is mainly lifted out of cmd.py in Python 2.2, with minor
 changes. Licensing should therefore be under the standard Python terms.  For
 details on the PSF (Python Software Foundation) standard license, see:
 
-http://www.python.org/2.2.3/license.html
-
-$Id: Debugger.py 2261 2007-04-20 17:50:00Z jdh2358 $"""
+http://www.python.org/2.2.3/license.html"""
 
 #*****************************************************************************
 #
-# Since this file is essentially a modified copy of the pdb module which is
-# part of the standard Python distribution, I assume that the proper procedure
-# is to maintain its copyright as belonging to the Python Software Foundation
-# (in addition to my own, for all new code).
+#       This file is licensed under the PSF license.
 #
 #       Copyright (C) 2001 Python Software Foundation, www.python.org
 #       Copyright (C) 2005-2006 Fernando Perez. <fperez@colorado.edu>
 #
-#  Distributed under the terms of the BSD License.  The full license is in
-#  the file COPYING, distributed as part of this software.
 #
 #*****************************************************************************
 
-from IPython import Release
-__author__  = '%s <%s>' % Release.authors['Fernando']
-__license__ = 'Python'
-
 import bdb
 import cmd
 import linecache
@@ -44,23 +33,25 @@ import sys
 
 from IPython import PyColorize, ColorANSI, ipapi
 from IPython.genutils import Term
-from IPython.excolors import ExceptionColors
+from IPython.excolors import exception_colors
 
 # See if we can use pydb.
 has_pydb = False
 prompt = 'ipdb> '
-try:
-    import pydb
-    if hasattr(pydb.pydb, "runl") and pydb.version>'1.17':
-        # Version 1.17 is broken, and that's what ships with Ubuntu Edgy, so we
-        # better protetct against it.
-        has_pydb = True
-        from pydb import Pdb as OldPdb
-except ImportError:
-    pass
+#We have to check this directly from sys.argv, config struct not yet available
+if '-pydb' in sys.argv:
+    try:        
+        import pydb
+        if hasattr(pydb.pydb, "runl") and pydb.version>'1.17':
+            # Version 1.17 is broken, and that's what ships with Ubuntu Edgy, so we
+            # better protect against it.
+            has_pydb = True
+    except ImportError:
+        print "Pydb (http://bashdb.sourceforge.net/pydb/) does not seem to be available"
 
 if has_pydb:
     from pydb import Pdb as OldPdb
+    #print "Using pydb for %run -d and post-mortem" #dbg
     prompt = 'ipydb> '
 else:
     from pdb import Pdb as OldPdb
@@ -213,7 +204,7 @@ class Pdb(OldPdb):
 
             # Create color table: we copy the default one from the traceback
             # module and add a few attributes needed for debugging
-            self.color_scheme_table = ExceptionColors.copy()
+            self.color_scheme_table = exception_colors()
 
             # shorthands 
             C = ColorANSI.TermColors
@@ -260,8 +251,7 @@ class Pdb(OldPdb):
 
             # Create color table: we copy the default one from the traceback
             # module and add a few attributes needed for debugging
-            ExceptionColors.set_active_scheme(color_scheme)
-            self.color_scheme_table = ExceptionColors.copy()
+            self.color_scheme_table = exception_colors()
 
             # shorthands 
             C = ColorANSI.TermColors
@@ -336,6 +326,12 @@ class Pdb(OldPdb):
         #frame, lineno = frame_lineno
         print >>Term.cout, self.format_stack_entry(frame_lineno, '', context)
 
+        # vds: >>
+        frame, lineno = frame_lineno
+        filename = frame.f_code.co_filename
+        __IPYTHON__.hooks.synchronize_with_editor(filename, lineno, 0)
+        # vds: <<
+
     def format_stack_entry(self, frame_lineno, lprefix=': ', context = 3):
         import linecache, repr
         
@@ -499,6 +495,12 @@ class Pdb(OldPdb):
             last = first + 10
         self.print_list_lines(self.curframe.f_code.co_filename, first, last)
 
+        # vds: >>
+        lineno = first
+        filename = self.curframe.f_code.co_filename
+        __IPYTHON__.hooks.synchronize_with_editor(filename, lineno, 0)
+        # vds: <<
+
     do_l = do_list
 
     def do_pdef(self, arg):
