@@ -40,11 +40,6 @@ import UserDict
 import warnings
 import glob
 
-try:
-    set
-except NameError:
-    from sets import Set as set
-
 def gethashfile(key):
     return ("%02x" % abs(hash(key) % 256))[-2:]
 
