@@ -1,7 +1,9 @@
 # -*- coding: utf-8 -*-
 """Support for wildcard pattern matching in object inspection.
 
-$Id: OInspect.py 608 2005-07-06 17:52:32Z fperez $
+Authors
+-------
+- Jörgen Stenarson <jorgen.stenarson@bostream.nu>
 """
 
 #*****************************************************************************
@@ -11,10 +13,6 @@ $Id: OInspect.py 608 2005-07-06 17:52:32Z fperez $
 #  the file COPYING, distributed as part of this software.
 #*****************************************************************************
 
-from IPython import Release
-__author__  = "Jörgen Stenarson <jorgen.stenarson@bostream.nu>"
-__license__ = Release.license
-
 import __builtin__
 import exceptions
 import pdb
